/*
  Copyright <2018-2024> <scott.e.graves@protonmail.com>

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#ifndef INCLUDE_UTILS_ERROR_UTILS_HPP_
#define INCLUDE_UTILS_ERROR_UTILS_HPP_

#include "types/repertory.hpp"

namespace repertory::utils::error {
void raise_error(std::string function, std::string_view msg);

void raise_error(std::string function, const api_error &err,
                 std::string_view msg);

void raise_error(std::string function, const std::exception &exception,
                 std::string_view msg);

void raise_error(std::string function, std::int64_t err, std::string_view msg);

void raise_error(std::string function, const json &err, std::string_view msg);

void raise_error(std::string function, const api_error &err,
                 std::string_view file_path, std::string_view msg);

void raise_error(std::string function, std::int64_t err,
                 std::string_view file_path, std::string_view msg);

void raise_error(std::string function, const std::exception &exception,
                 std::string_view file_path, std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          const api_error &err, std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          const std::exception &exception,
                          std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          std::int64_t err, std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          const json &err, std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          std::string_view source_path, const api_error &err,
                          std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          std::string_view source_path, std::int64_t err,
                          std::string_view msg);

void raise_api_path_error(std::string function, std::string_view api_path,
                          std::string_view source_path,
                          const std::exception &exception,
                          std::string_view msg);

void raise_url_error(std::string function, std::string_view url, CURLcode err,
                     std::string_view msg);

void raise_url_error(std::string function, std::string_view url,
                     std::string_view source_path,
                     const std::exception &exception, std::string_view msg);

} // namespace repertory::utils::error

#endif // INCLUDE_UTILS_ERROR_UTILS_HPP_
